/***************************************************************************/
/****				   Header File for SAY Version 1.82					****/
/****			Copyright (C) 1993,1995 NOWSOFT , YUTAKA AOKI			****/
/***************************************************************************/

#ifndef __SAY_H__
#define __SAY_H__

#ifdef __cplusplus
extern "C" {
#endif

/****					Definitions										****/


#define		SAY_SOFTINTNO	0x41
#define		SAY_VERSION		182
#define		DATA_VERSION	2

enum
{
	CLEN10=96,CLEN11,CLEN12,CLEN13,CLEN14,CLEN15,CLEN16,CLEN17,CLEN18,CLEN19,
	CLEN1A,CLEN1B,CLEN1C,CLEN1D,CLEN1E,CLEN1F,
	CREST1 , CREST2 ,
	CNOP ,
	CLEN2 ,
	CEND ,
	CJR , CMAINLOOP ,
	CSYNC , CWAIT , CEXTSYNC , CEXTWAIT , CCOMSYNC ,
	CLTOP , CLBRK , CLBACK , CCALL , CRET ,
	CFADE ,
	CTIMBRE , CENV , CENVOFF ,
	CTIE1 , CTIE2 ,
	CTEMPO ,
	CRILEVEL , CVOL ,
	CALEVEL , CMLEVEL ,
	CLEVEL ,
	CVOLP , CVOLM , CVOLPM ,
	CSETQ1 , CSETQ2 , CSETQ3 ,
	CTUNE , CDT ,
	CPITCH1 , CPITCH2 , CPITCH3 ,
	CTLFO0 , CTLFO1 , CTLFO2 ,
	CVLFO ,
	CTLFOSW , CVLFOSW ,
	CSETTN , CSETNOISE ,
	CWRITE1 ,
	CWRITE2 ,
	CPAN ,
	CNOISEMODE
};

enum {
	S_ER_OK = 0 ,
	S_ER_ILL_FUNC ,
	S_ER_BUF_OVERFLOW ,
	S_ER_TOO_MANY_SNDFILE ,
	S_ER_SND_FILE_NOT_FOUND	,
	S_ER_SND_FILE_TOO_LARGE	,
	S_ER_SND_FILE_VER_ERR ,
	S_ER_PCM_FILE_NOT_FOUND ,
	S_ER_PCM_FILE_TOO_LARGE ,
	S_ER_PCM_FILE_DENIED ,
	S_ER_PCM_EMS_OVERFLOW ,
	S_ER_NOT_SND_HND ,
	S_ER_NOT_TASK_HND
};

/*****						Type Definitions							*****/

typedef unsigned char TSoundData;

typedef struct
{
	unsigned char	R30;
	unsigned char	R40;
	unsigned char	R50;
	unsigned char	R60;
	unsigned char	R70;
	unsigned char	R80;
} TOpReg;

typedef struct
{
	unsigned int	Ml:4;
	unsigned int	Dt:3;
	unsigned int	:1;
	
	unsigned int	Tl:7;
	unsigned int	:1;
	
	unsigned int	Ar:5;
	unsigned int	:1;
	unsigned int	Ks:2;
	
	unsigned int	Dr:5;
	unsigned int	:3;
	
	unsigned int	Sr:5;
	unsigned int	:3;
	
	unsigned int	Rr:4;
	unsigned int	Sl:4;
} TOpName;

typedef union
{
	TOpReg		Reg;
	TOpName		Name;
} TOperator;

typedef struct {
	unsigned char	RB0;
	TOperator		Op[4];
} TFMEnvelope;

typedef struct {
	unsigned char	Sv , Ar , Dr , Sl , Sr , Rr;
} TPSGEnvelope;

typedef struct {
	unsigned char	Sv , Ar , Dr , Sl , Sr , Rr;
} TPCMEnvelope2;

typedef struct {
	unsigned char	Filename[8];
} TPCMEnvelope1;

typedef struct {
	unsigned char	Flg;		/*** Flg( 1:Use )	***/
	
	unsigned char	Kind;		/*** Kind	***/
	unsigned char	Interval;
	int				Step;
	unsigned char	Delay;		/*** Delay	***/
	unsigned char	Depth;		/*** Depth	***/
	
	unsigned char	UpDown;		/*** Flg( 0:Up , 1:Down )	***/
	unsigned char	Interval2;
	long			Step2;
	unsigned char	Delay2;		/*** Delay Counter	***/
	unsigned char	Depth2;		/*** Depth Counter	***/
} TTLFO;

typedef struct {
	unsigned char	Flg;		/*** Flg( 1:Use )	***/
	
	unsigned char	Kind;		/*** Kind	***/
	unsigned char	Interval;
	int				Step;
	unsigned char	Delay;		/*** Delay	***/
	unsigned char	Depth;		/*** Depth	***/
	
	unsigned char	UpDown;		/*** Flg( 0:Up , 1:Down )	***/
	unsigned char	Interval2;
	int				Swing;		/*** Swing/256 	***/
	unsigned char	Delay2;		/*** Delay Counter	***/
	unsigned char	Depth2;		/*** Depth Counter	***/
} TVLFO;

typedef enum
{
	ST_END ,
	ST_ON ,
	ST_ERR
} TStatusFlg;

typedef struct {
	unsigned char	Flg;
	
	TSoundData		near *Top;
	TSoundData		near *Poi;
	TSoundData		near *StackBottom;
	TSoundData		near *Stack;
	
	unsigned char	KeyOnFlg;
	
	unsigned char	AllTime;
	unsigned char	OnTime;
	
	unsigned char	QFlg;	/*** Q Flag ( 0 , 1 , 2 ) ***/
	unsigned int	Q;		/*** GateTime : Point Number(0-2) or Counts ***/
	
	unsigned char	CurLen;
	
	unsigned int	Tone1;	/* 0ffffh:Rest   , Other:Main Note Code			*/
	unsigned char	Oct1;
	unsigned int	Tone2;
	
	long			Freq;	/*** Current Frequency ( Unstatic )		***/
	long			Freq1;	/*** Main Frequency ( Static )			***/
	long			Freq2;	/*** Destination Frequency ( Static )	***/
	
	unsigned char	PitchFlg;
	unsigned char	PitchDelay;			/*** Delay Counter	***/
	unsigned char	PitchInterval;		/*** Const		***/
	unsigned char	PitchInterval2;		/*** Counter	***/
	long			PitchStep;
	
	int				Tune;	/*** ϒ ( Note Code ) ***/
	char			Dt;		/*** Detune ***/
	
	unsigned char	LFOMode;
	
	TTLFO			TLFO[3];
	TVLFO			VLFO;
	
	unsigned char	TieFlg;		/*** Tie Flag ( 0 , 1 , 2 )	***/
	
	unsigned char	Volume;		/***	Add this to Tl		***/
	
	unsigned char	ChangeFlg;
	
	TFMEnvelope		near *Env;
	unsigned char	LR_AMS_PMS;
} TFMStatus;

typedef struct {
	unsigned char	Flg;
	
	TSoundData		near *Top;
	TSoundData		near *Poi;
	TSoundData		near *StackBottom;
	TSoundData		near *Stack;
	
	unsigned char	KeyOnFlg;
	
	unsigned char	AllTime;
	unsigned char	OnTime;
	
	unsigned char	QFlg;	/*** Q Flag ( 0 , 1 , 2 ) ***/
	unsigned int	Q;		/*** GateTime : Point Number(0-2) or Counts ***/
	
	unsigned char	CurLen;
	
	unsigned int	Tone1;	/* 0ffffh:Rest   , Other:Main Note Code			*/
	unsigned char	Oct1;
	unsigned int	Tone2;
	
	long			Freq;	/*** Current Frequency ( Unstatic )		***/
	long			Freq1;	/*** Main Frequency ( Static )			***/
	long			Freq2;	/*** Destination Frequency ( Static )	***/
	
	unsigned char	PitchFlg;
	unsigned char	PitchDelay;			/*** Delay Counter	***/
	unsigned char	PitchInterval;		/*** Const		***/
	unsigned char	PitchInterval2;		/*** Counter	***/
	long			PitchStep;
	
	int				Tune;	/*** ϒ ( Note Code ) ***/
	char			Dt;		/*** Detune ***/
	
	unsigned char	LFOMode;
	
	TTLFO			TLFO[3];
	TVLFO			VLFO;
	
	unsigned char	TieFlg;		/*** Tie Flag ( 0 , 1 , 2 )	***/
	
	unsigned char	MaxLevel;	/***	Integer Number(0-15)	***/
	
	TPSGEnvelope	near *Env;
	
	unsigned char	Level;		/***	Integer Number(0-15)	***/
	unsigned char	EnvFlg;
	unsigned char	LogicalLevel;
	
	unsigned char	AutoMaxLogicalLevelFlg;
	unsigned char	MaxLogicalLevel;
	
	unsigned char	NoiseMode;
} TPSGStatus;

typedef struct {
	unsigned char	Flg;
	
	TSoundData		near *Top;
	TSoundData		near *Poi;
	TSoundData		near *StackBottom;
	TSoundData		near *Stack;
	
	unsigned char	KeyOnFlg;
	
	unsigned char	AllTime;
	unsigned char	OnTime;
	
	unsigned char	QFlg;	/*** Q Flag ( 0 , 1 , 2 ) ***/
	unsigned int	Q;		/*** GateTime : Point Number(0-2) or Counts ***/
	
	unsigned char	CurLen;
	
	unsigned char	Instrument;
	
	unsigned char	ChangeFlg;
	
	unsigned char	LR_ILevel[6];
	
	unsigned char	Volume;
} TRythmStatus;

typedef struct {
	unsigned char	Flg;
	
	TSoundData		near *Top;
	TSoundData		near *Poi;
	TSoundData		near *StackBottom;
	TSoundData		near *Stack;
	
	unsigned char	KeyOnFlg;
	
	unsigned char	AllTime;
	unsigned char	OnTime;
	
	unsigned char	QFlg;	/*** Q Flag ( 0 , 1 , 2 ) ***/
	unsigned int	Q;		/*** GateTime : Point Number(0-2) or Counts ***/
	
	unsigned char	CurLen;
	
	unsigned int	Tone1;	/* 0ffffh:Rest   , Other:Main Note Code			*/
	unsigned char	Oct1;
	unsigned int	Tone2;
	
	long			Freq;	/*** Current Frequency ( Unstatic )		***/
	long			Freq1;	/*** Main Frequency ( Static )			***/
	long			Freq2;	/*** Destination Frequency ( Static )	***/
	
	unsigned char	PitchFlg;
	unsigned char	PitchDelay;			/*** Delay Counter	***/
	unsigned char	PitchInterval;		/*** Const		***/
	unsigned char	PitchInterval2;		/*** Counter	***/
	long			PitchStep;
	
	int				Tune;	/*** ϒ ( Note Code ) ***/
	char			Dt;		/*** Detune ***/
	
	unsigned char	LFOMode;
	
	TTLFO			TLFO[3];
	TVLFO			VLFO;
	
	unsigned char	TieFlg;		/*** Tie Flag ( 0 , 1 , 2 )	***/
	
	unsigned char	MaxVolume;		/*	Max Volume	*/
	
	TPCMEnvelope2	near *Env2;
	
	unsigned char	Volume;			/*	Current Volume	*/
	unsigned char	EnvFlg;
	unsigned char	LogicalLevel;
	
	unsigned char	AutoMaxLogicalLevelFlg;
	unsigned char	MaxLogicalLevel;
	
	unsigned int	PCMEnv1Off;
	unsigned int	PCMEnv1Page;
	
	unsigned long	PCMPos;
	
} TPCMStatus;

typedef struct
{
	unsigned char		MCBFlg;
	unsigned int		MCBSize;
	
	unsigned char		MixFlg;
	unsigned char		PriorityOrder;
	
	unsigned int		SndID;
	
	unsigned char		IntAccess;
	
	unsigned char		MainLoopCount;
	unsigned char		MainLoopFade;
	
	unsigned int		WaitFlg;
	
	unsigned char		UserTempo;
	unsigned int		NextPeriod;
	
	unsigned char		ActivePart;
	unsigned char		PCMActivePart;
	
	unsigned char		FadeCount0;
	unsigned char		FadeCount;
	unsigned char		FadeLevel;
	unsigned char		Dummy;
	
	unsigned char		ComSyncFlg;
	unsigned char		ExtSyncFlg;
	
	unsigned char		ToneNoise;
	unsigned char		Noise;
	
	unsigned char		EnvDataSu;
	unsigned char		FMPartSu;
	unsigned char		PSGPartSu;
	unsigned char		RythmPartSu;
	unsigned char		PCMPartSu;
	unsigned char		TailDataSu;
	
	unsigned char		near *TailDataTop;
	unsigned char		PCMEnv1Su;
	
	unsigned char		near *LenTabTop;
	TFMEnvelope			near *FMEnvTop;
	TPSGEnvelope		near *PSGEnvTop;
	TPCMEnvelope1		near *PCMEnv1Top;
	TPCMEnvelope2		near *PCMEnv2Top;
	
	TFMStatus			near *FMStTop;
	TPSGStatus			near *PSGStTop;
	TRythmStatus		near *RythmStTop;
	TPCMStatus			near *PCMStTop;
} TUserInfo;

typedef struct
{
	unsigned int		PartMaskFlg;
	unsigned int		CurrentPart;
	
	unsigned char		CurrentPort;
	
	unsigned char		FMWeek;
	unsigned char		PSGWeek;
	unsigned char		RhythmWeek;
	unsigned char		PCMWeek;
	
	unsigned char		MonoFlg;
	
	unsigned char		ExtTimerFlg;
	unsigned char		SkipTempo;
	
	unsigned char		Volume;
	unsigned char		Dummy;
	
	unsigned char		SndDataSu;
	unsigned int		SndDataSeg[32];
	
	unsigned char		SndPartFlg[26];
	unsigned int		SndLastSeg[26];
	
	unsigned int		IDCnt;
} TSndInfo;

typedef struct
{
	unsigned char		SAYTitle[9];
	unsigned char		SAYVersion;
	unsigned char		DataVersion;
	
	unsigned int		SAYPSP;
	
	unsigned char		IntFlg;
	
	/*|JX^bNgpvO*/
	unsigned char		LocalStackUse;
	
	/*^C}|ɂ 0 ȊO ɂȂAłȂɂ́A0 ɂȂĂB*/
	unsigned char		TimerIntService;
	/*O 0 ɂĂƁA^C}|x 0 ȊO ɂȂB
	@pƃx|^|ȂǂŐmȓƂ邱ƂoB*/
	unsigned char		TimerIntAccess;
	
	unsigned char		OPNIntNo;
	void far interrupt	( *OPNIntVect )( void );
	void far interrupt	( *Int28Vect )( void );
	
	unsigned int		I8259CW0;
	unsigned int		I8259CW1;
	unsigned char		I8259MaskBit;
	
	void far interrupt	( *LastSoftIntVect )( void );
	void far interrupt	( *LastOPNIntVect )( void );
	void far interrupt	( *LastInt28Vect )( void );
	
	TSndInfo			far *SndInfoPoi;
	
	unsigned int		SndBufSeg;
	unsigned int		SndBufSize;
	unsigned int		LocalStackSeg;
	unsigned int		LocalStackSize;
	
	unsigned char		GaijiLoadFlg;
	
	unsigned char		OPNKind;
	unsigned char		PCMKind;
	
	unsigned int		OPNPortNo;
	
	unsigned char		Int28Flg;
	
	unsigned int		PCMBufSeg;
	
	unsigned int		PCMUseCnt;
	
	unsigned char		PCMRate;
	
	unsigned char		FIFOLeft0;
	unsigned char		FIFOLeft;
	
	unsigned int		PCMSendBytes;
	
	unsigned char		EMSExistFlg;
	unsigned int		EMSHandle;
	unsigned int		EMSPageMax;
	
	unsigned int		EMSRestTopOff;
	unsigned int		EMSRestTopPage;
	
	void				huge ( *LinkJumpAddr )( void );
	
	unsigned int		ErrNo;
} TDrvInfo;

enum {
	SAY_INIT=0 ,
	SAY_ALL_STOP ,
	SAY_ALL_CONT ,
	SAY_SND_START ,
	SAY_SND_FILE_LOAD ,
	SAY_SND_STOP ,
	SAY_SND_CONT ,
	SAY_SET_PART_MASK ,
	SAY_SET_EXT_TEMPO ,
	SAY_OFF_EXT_TEMPO ,
	SAY_SET_TEMPO ,
	SAY_SET_VOL ,
	SAY_OPNA_WRITE ,
	SAY_OPNB_WRITE ,
	SAY_OPNA_READ ,
	SAY_FADE_START ,
	SAY_FADE_STOP ,
	SAY_FADE_CLEAR ,
	SAY_FADE_WAIT ,
	SAY_TASK_RESERVE ,
	SAY_TASK_REMOVE ,
	SAY_TASK_SEARCH ,
	SAY_TASK_ALL_CLR
};

typedef struct
{
	unsigned char	DataVersion;
	unsigned char	DataFlg;
	
	unsigned char	EnvDataSu;
	unsigned char	FMDataSu;
	unsigned char	PSGDataSu;
	unsigned char	RythmDataSu;
	unsigned char	PCMDataSu;
	unsigned char	TailDataSu;
	
	unsigned int	Addr[5+6+3+1+16+1];
} TSAYHeader;

#ifdef __cplusplus
}
#endif

#endif
